<?php

// Authentification

if (@file_exists('inc/auth.inc.php') && @include_once 'inc/auth.inc.php');
else exit;

// General options

$ezin_opts['dbh'] = ezin_open_db_connection();
$ezin_opts['tb']  = $ezin_sys['db']['tables']['articles'];
$ezin_opts['key'] = 'id';
$ezin_opts['key_type'] = 'int';
$ezin_opts['sort_field'] = 'id';
$ezin_opts['inc'] = 15;
$ezin_opts['options'] = $ezin_USER['role_articles'];
$ezin_opts['multiple'] = '4';
$ezin_opts['navigation'] = $ezin_cfg['admin_navigation'].$ezin_cfg['admin_buttons_pos'];
$ezin_opts['display'] = array('query' => false, 'sort'  => false, 'time'  => false);
$ezin_opts['url'] = array('images' => 'icons/');
$ezin_opts['sort_field'] = array('-datetime', '-id');
$ezin_opts['language'] = 'EN';

// Triggers

$ezin_opts['triggers']['insert']['after'] = 'inc-db/triggers/articles.after';
$ezin_opts['triggers']['update']['after'] = 'inc-db/triggers/articles.after';
$ezin_opts['triggers']['delete']['after'] = 'inc-db/triggers/articles.after';

// Field definitions

$ezin_opts['fdd']['id'] = array(
		'name'=>'ID',
		'select'=>'T',
		'maxlen'=>3,
		'sort'=>true,
		'default'=>'',
		'options'=>'LDVFPCR',
		'URLprefix'=>$ezin_cfg['url'].'?ezin_article_id=',
		'URLtarget'=>'_blank',
		/*'sql|LF'=>'IF(TRIM(PMEtable0.access_key) != "", CONCAT(PMEtable0.id, " / ", PMEtable0.access_key), PMEtable0.id)'*/
		);

$ezin_opts['fdd']['access_key'] = array(
		'name'=>Platon::htmlspecialchars2('Access key'),
		'name|LF'=>'Key',
		'sort'=>true,
		'select'=>'T',
		'options'=>'ACPDV',
		);

$ezin_opts['fdd']['template_key'] = array(
		'name'=>Platon::htmlspecialchars2('Template key'),
		'name|LF'=>'Key',
		'sort'=>true,
		'select'=>'T',
		'options'=>'ACPDV',
		);

$ezin_opts['fdd']['status'] = array(
		'name'=>'Status',
		'name|LF'=>'S',
		'select'=>'D',
		'values'=>array(),
		'values2'=> array(
			'showed'        => '<b>showed</b> (default)',
			'partly_showed' => '<b>partly showed</b> (hidden from global sections)',
			'hidden'        => '<b>not showed</b> (hidden, but accessible)',
			'disabled'      => '<b>disabled</b> (unaccessible)'),
		'values2|LF'=>array(
			''              => '<code>S?</code>',
			'showed'        => '<code>S</code>',
			'partly_showed' => '<code>s</code>',
			'hidden'        => '<code>H</code>',
			'disabled'      => '<code>D</code>'),
		'sort'=>true
		);

$ezin_opts['fdd']['bind_lang'] = array(
		'name'=>Platon::htmlspecialchars2('Language bind'),
		'select'=>'D',
		'sort'=>true,
		'options' => 0 && is_array($ezin_cfg['languages']) ? 'ACPVD' : ''
		);

$ezin_opts['fdd']['bind_theme'] = array(
		'name'=>Platon::htmlspecialchars2('Theme bind'),
		'select'=>'D',
		'sort'=>true,
		'options' => 0 && 1 ? 'ACPVD' : ''
		);

$ezin_opts['fdd']['accesses_count'] = array(
		'name'=>Platon::htmlspecialchars2('Accesses count'),
		'options'=>'VDACP'.'HR',
		'default'=>'0'
		);

$ezin_opts['fdd']['sessions_count'] = array(
		'name'=>Platon::htmlspecialchars2('Sessions count'),
		'options'=>'VDACP'.'HR',
		'default'=>'0'
		);

$ezin_opts['fdd']['message_count'] = array(
		'name'=>Platon::htmlspecialchars2('Message count'),
		'options'=>'VDACP'.'HR',
		'default'=>'0'
		);


$ezin_opts['fdd']['author_id'] = array(
		'name'=>'Author',
		'select'=>'T',
		'sort'=>true,
		'required'=>true,
		'values'=>array(
			'table' => $ezin_sys['db']['tables']['authors'],
			'column' => 'id',
			'description' => 'name')
		);

$ezin_opts['fdd']['section_id'] = array(
		'name'=>'Section',
		'select'=>'T',
		'sort'=>true,
		'required'=>true,
		'values'=>array(
			'table' => $ezin_sys['db']['tables']['sections'],
			'column' => 'id',
			'description' => array(
				'columns' => array('name', 'id'),
				'divs'    => array(' (', ')')))
		);

$ezin_opts['fdd']['datetime'] = array(
		'name'=>Platon::htmlspecialchars2('Date & Time'),
		'select'=>'T',
		'sort'=>true,
		'required'=>true,
		'mask'=>'%.10s',
		'default'=>date('Y-m-d H:i:s')
		);

$ezin_opts['fdd']['atitle'] = array(
		'name'=>Platon::htmlspecialchars2('Annotation title'),
		'select'=>'T',
		'maxlen'=>255,
		'sort'=>true,
		'options'=>'ACPVD',
		);

$ezin_opts['fdd']['title'] = array(
		'name'=>'Title',
		'select'=>'T',
		'maxlen'=>255,
		'sort'=>true,
		'trimlen|LF'=>35,
		'sql|LF'=>'IF(TRIM(title) != "", title, atitle)'
		);

$ezin_opts['fdd']['hometext_flags'] = array(
		'name'=>Platon::htmlspecialchars2('Anotation flags'),
		'select'=>'T',
		'options'=>'HCAP'
		);

$ezin_opts['fdd']['bodytext_flags'] = array(
		'name'=>Platon::htmlspecialchars2('Text flags'),
		'select'=>'T',
		'options'=>'HCAP'
		);

$ezin_opts['fdd']['hometext'] = array(
		'name'=>'Anotation',
		'select'=>'T',
		'textarea'=>array(
			'rows'=>8,
			'cols'=>65,
			),
		'options'=>'ACPV'
		);

$ezin_opts['fdd']['bodytext'] = array(
		'name'=>Platon::htmlspecialchars2('Article text'),
		'select'=>'T',
		'textarea'=>array(
			'rows'=>15,
			'cols'=>65,
			'wrap'=>'virtual'
			),
		'options'=>'ACPV'
		);

// Execution code

if (! ezin_admin_check_list_privileges($ezin_opts['options'])) {
	require 'inc-html/permdenied.html';
} else {
	/* if (! ezin_admin_check_privileges($ezin_opts['options'],
	   isset($author_id) && $author_id == $ezin_USER['author_id'])) {
	   require 'inc-html/permdenied.html';
	   } */

	require_once 'phpMyEdit/phpMyEdit.class.php';

	$ezin_opts['cgi']['overwrite'][phpMyEdit::get_default_cgi_prefix('data').'hometext_flags'] = ezin_get_special_tags_flags(phpMyEdit::get_data_cgi_var('hometext'));
	$ezin_opts['cgi']['overwrite'][phpMyEdit::get_default_cgi_prefix('data').'bodytext_flags'] = ezin_get_special_tags_flags(phpMyEdit::get_data_cgi_var('bodytext'));


	new phpMyEdit($ezin_opts);
}

?>
