<?php

/*
 * phpMyEzin - e-zin content management system
 *
 * error.inc.php - error handling
 * ____________________________________________________________
 *
 * Developed by Ondrej Jombik <nepto@platon.sk>
 * Copyright (c) 2001-2004 Platon SDG, http://platon.sk/
 * All rights reserved.
 *
 * See README file for more information about this software.
 * See COPYING file for license information.
 *
 * Download the latest version from
 * http://platon.sk/projects/phpMyEzin/
 */

/* $Platon: $ */

require_once 'config/loader.inc.php';
Platon::redirect_test(basename(__FILE__), $ezin_cfg['url']);
require_once 'HTML/Template/Platon.php';

function ezin_php_error_handler($errno, $errstr, $errfile, $errline) /* {{{ */
{
	if (! ($errno & error_reporting())) {
		return;
	}
	if (0) {
		echo "<hr><b>ERROR:</b> $errno:$errstr:$errfile:$errline<br>\n";
		return;
	}
	$errortype = array (
			1    => 'error',
			2    => 'warning',
			4    => 'parsing error',
			8    => 'notice',
			16   => 'core error',
			32   => 'core warning',
			64   => 'compile error',
			128  => 'compile warning',
			256  => 'user error',
			512  => 'user warning',
			1024 => 'user notice'
			);
	$err_ar = array(
			'error_reporting' => error_reporting(),
			'error_file'      => trim($errfile),
			'error_line'      => trim($errline),
			'error_type'      => $errortype[$errno],
			'error_code'      => trim($errno),
			'error_message'   => trim($errstr)
			);

	@ob_end_clean();
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_cgi;

	$tmpl = ezin_load_template($ezin_sys['tmpl']['error'], 1);	
	if ($tmpl != false) {
		$tmpl = new HTML_Template_Platon($ezin_sys['tmpl']['error']);
	}
	if ($tmpl == false || $tmpl->isVar('content')) {
		$content  = '';
		$content .= '<table' . ($tmpl == false ? ' bgcolor="lightblue"' : '') . '>';
		$content .= '<tr><th colspan="2" align="center">';
		$content .= 'An error occured';
		$content .= '</th></tr>';
		$content .= '<tr><td colspan="2">&nbsp;</td></tr>'."\n";
		foreach ($err_ar as $key => $val) {
			$content .= '<tr><td>';
			$content .= '<b>' . htmlspecialchars($key) . '</b>';
			$content .= '</td><td>';
			$content .= htmlspecialchars($val);
			$content .= '</td></tr>'."\n";
		}
		$content .= '<tr><td colspan="2">&nbsp;</td></tr>';
		$content .= '<tr><td colspan="2" align="center">';
		$content .= 'Powered by';
		$content .= ' <a target="_blank" href="'.$ezin_sys['software']['url'].'">';
		$content .= $ezin_sys['software']['name'].'</a>';
		$content .= '</td></tr>';
		$content .= '</table>';
		if ($tmpl != false) {
			$tmpl->set('content', $content);
		}
	}
	if ($tmpl != false) {
		if (preg_match('|^.*index.php\(\d+\) : eval\(\)\'d code$|', $errfile)) {
			$err_ar['error_source'] = ezin_get_main_template_source($errline);
		} else {
			$err_ar['error_source'] = 'not available';
		}
		$tmpl->set($err_ar);
		ezin_substitute_basic_properties($tmpl);
		//echo(htmlspecialchars($tmpl->getCode()));
		@$tmpl->Output();
	} else {
		echo $content;
	}
	ezin_error_send_notify_email($err_ar);
	exit;
} /* }}} */

function ezin_pear_error_handler($err_obj) /* {{{ */
{
	$error_string = $err_obj->getMessage();
	// SENSITIVE!!! . ' | ' . $err_obj->getDebugInfo();
	trigger_error($error_string, E_USER_ERROR);
} /* }}} */

function ezin_get_main_template_source($line_around) /* {{{ */
{
	global $ezin_tmpl_main;
	$index   = $line_around - 10;
	$content = '';
	$code_ar = @$ezin_tmpl_main->getCode();
	$code_ar = preg_split('|[\n]|', $code_ar);
	$code_ar = array_slice($code_ar, $index - 1, 25);
	foreach ($code_ar as $val) {
		if (strlen($val) > 60) {
			$val = trim(str_replace(';', ";\n      ", $val));
			$val = trim(str_replace('.\'', "\n      .'", $val));
		}
		$val = sprintf('%04d: %s', $index, htmlspecialchars($val));
		if ($index == $line_around) {
			$val = "<b>$val</b>";
		}
		$content .= $val."\n";
		$index++;
	}
	return $content;
} /* }}} */

function ezin_error_send_notify_email($err_ar) /* {{{ */
{
	global $ezin_sys, $ezin_cfg;
	if (! isset($ezin_cfg['admin_email']) || strlen($ezin_cfg['admin_email']) <= 0) {
		return true;
	}
	$from     = '';
	$subject  = '';
	$encoding = '';
	$message  = '';
	$err_ar['error_source'] = Platon::undo_htmlspecialchars(@$err_ar['error_source']);	
	$err_ar['error_source'] = preg_replace('|<b>(.*)</b>|', '--> \\1 <--',
			$err_ar['error_source']);
	$tmpl = ezin_load_template($ezin_sys['tmpl']['e-mail']['error'], 1);	
	if ($tmpl != false) {
		$tmpl = new HTML_Template_Platon($ezin_sys['tmpl']['e-mail']['error']);
	}
	if ($tmpl == false || $tmpl->isVar('content')) {
		$message .= 'Dear administrator of ';
		$message .= $ezin_sys['software']['name']." on $ezin_cfg[name]\n\n";
		$message .= 'An error has occured on your portal at server time ';
		$message .= date($ezin_cfg['format_datetime']) . '.';
		$message .= "\nFull error report follows on the next lines.\n\n";
		foreach ($err_ar as $key => $val) {
			$message .= sprintf("* %s:%s%s\n", $key, $key == 'error_source' ? "\n\n" : ' ', $val);
		}
		$message .= "\nI wish you a fast and effective solution of this problem.";
		$message .= "\n\nYours,\n\n";
		$message .= $ezin_sys['software']['name']." on $ezin_cfg[name]\n\n";
		if ($tmpl != false) {
			$tmpl->set('content', $content);
		}
	}
	if ($tmpl != false) {
		$tmpl->set($err_ar);
		ezin_substitute_basic_properties($tmpl);
		ob_start();
		@$tmpl->Output();
		$message = ob_get_contents();
		ob_end_clean();
		$message = ezin_email_template_parse($message, $from, $subject, $encoding);
	}
	empty($subject) && $subject = sprintf('%s: error report', $ezin_cfg['name']);
	return ezin_email_send($ezin_cfg['admin_email'], $from, $subject, $encoding, $message);
} /* }}} */

error_reporting(E_ALL);
set_error_handler('ezin_php_error_handler');
require_once 'PEAR.php';
PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, 'ezin_pear_error_handler');

/* Modeline for ViM {{{
 * vim: set ts=4:
 * vim600: fdm=marker fdl=0 fdc=0:
 * }}} */

?>
