<?php

/*
 * phpMyEzin - e-zin content management system
 *
 * functions.inc.php - various function definitions
 * ____________________________________________________________
 *
 * Developed by Ondrej Jombik <nepto@platon.sk>
 * Copyright (c) 2001-2004 Platon SDG, http://platon.sk/
 * All rights reserved.
 *
 * See README file for more information about this software.
 * See COPYING file for license information.
 *
 * Download the latest version from
 * http://platon.sk/projects/phpMyEzin/
 */

/* $Platon: $ */

/*
 * Line wrap function
 */

function ezin_process_text_flags(&$str, $type, $special = 0) /* {{{ */
{
	global $ezin_cfg;
	$http_substs  = stristr(@$ezin_cfg['flags_'.$type], 'H') != false;
	$www_substs   = stristr(@$ezin_cfg['flags_'.$type], 'W') != false;
	$email_substs = stristr(@$ezin_cfg['flags_'.$type], 'E') != false;
	$line_wrap_p  = stristr(@$ezin_cfg['flags_'.$type], 'P') != false;
	$line_wrap_br = stristr(@$ezin_cfg['flags_'.$type], 'B') != false;

	/* Links substitution code borrowed from phpBB2, http://www.phpbb.com/
	   See make_clickable() in includes/bbcode.php for more info. */

	// pad it with a space so we can match things at the start of the 1st line.
	$str = ' ' . $str;

	$http_substs && $str = preg_replace("#([\n ])([a-z]+?)://([a-z0-9\-\.,\?!%\*_\#:;~\\&$@\/=\+]+)#i", "\\1<a href=\"\\2://\\3\" target=\"_blank\">\\2://\\3</a>", $str);
	$www_substs && $str = preg_replace("#([\n ])www\.([a-z0-9\-]+)\.([a-z0-9\-.\~]+)((?:/[a-z0-9\-\.,\?!%\*_\#:;~\\&$@\/=\+]*)?)#i", "\\1<a href=\"http://www.\\2.\\3\\4\" target=\"_blank\">www.\\2.\\3\\4</a>", $str);
	$email_substs && $str = preg_replace("#([\n ])([a-z0-9\-_.]+?)@([\w\-]+\.([\w\-\.]+\.)?[\w]+)#i", "\\1<a href=\"mailto:\\2@\\3\">\\2@\\3</a>", $str);
	
	if ($line_wrap_p) {
		if ($special) {
			$str = preg_replace('/[\n\r]{3,}/m', "\n<br><br>", $str);
			$str = preg_replace('/[\n]{2,}/m', "\n<br><br>", $str);
		} else {
			$str = preg_replace('/[\n\r]{3,}/m', "\n<p>", $str);
		}
	}

	if ($line_wrap_br) {
		$str = nl2br($str);
	}

	// Remove our padding
	$str = substr($str, 1);
} /* }}} */

function ezin_get_special_tags_flags($str) /* {{{ */
{
	$ret = '';
	preg_match('|<flash\s[^>]+>|i', $str)           && $ret .= 'f';
	preg_match('|<image\s[^>]+>|i', $str)           && $ret .= 'i';
	preg_match('|<link\s[^>]+>|i', $str)            && $ret .= 'L';
	preg_match('|<extern-link\s[^>]+>|i', $str)     && $ret .= 'E';
	preg_match('|<repository-link\s[^>]+>|i', $str) && $ret .= 'E';
	preg_match('|<image-link\s[^>]+>|i', $str)      && $ret .= 'I';
	preg_match('|<mail-link\s[^>]+>|i', $str)       && $ret .= 'M';
	preg_match('|<section-link\s[^>]+>|i', $str)    && $ret .= 'S';
	preg_match('|<article-link\s[^>]+>|i', $str)    && $ret .= 'A';
	return $ret;
} /* }}} */

function ezin_handle_special_tags(&$str, $mask = 'fiLERIMSA') /* {{{ */
{
	static $ezin_url       = null;
	static $repository_url = null;
	static $link_file_type = null;
	if ($ezin_url == null) {
		global $ezin_cfg, $ezin_sys;
		$ezin_url       = Platon::append_slash($ezin_cfg['url']);
		$repository_url = Platon::append_slash($ezin_url.$ezin_sys['dir']['upload']);
		$link_file_type = $ezin_cfg['link_file_type'];
	}
	$pattern     = array();
	$replacement = array();
	if (strpos($mask, 'f') !== false) { /* {{{ */
		$pattern[] = '|<flash\s+(\w{3,6}://[^>\s]+)([^>]*)>((?:(?!<flash).)*?)</flash>|si';
		$pattern[] = '|<flash\s+([^>\s]+)([^>]*)>((?:(?!<flash).)*?)</flash>|si';
		$pattern[] = '|<flash\s+(\w{3,6}://[^>\s]+)([^>]*)>|i';
		$pattern[] = '|<flash\s+([^>\s]+)([^>]*)>|i';
		$replacement[] = '<object type="application/x-shockwave-flash" data="\\1"\\2>'."\n"
			.'<param name="movie" value="\\1"><embed src="\\1"\\2>\\3</object>';
		$replacement[] = '<object type="application/x-shockwave-flash" data="'.$repository_url.'\\1"\\2>'."\n"
			.'<param name="movie" value="'.$repository_url.'\\1"><embed src="'.$repository_url.'\\1"\\2>\\3</object>';
		$replacement[] = '<object type="application/x-shockwave-flash" data="\\1"\\2>'."\n"
			.'<param name="movie" value="\\1"><embed src="\\1"\\2></object>';
		$replacement[] = '<object type="application/x-shockwave-flash" data="'.$repository_url.'\\1"\\2>'."\n"
			.'<param name="movie" value="'.$repository_url.'\\1"><embed src="'.$repository_url.'\\1"\\2></object>';
	} /* }}} */
	if (strpos($mask, 'i') !== false) { /* {{{ */
		$pattern[] = '|<image\s+(\w{3,6}://[^>\s]+)([^>]*)>((?:(?!<image).)*?)</image>|si';
		$pattern[] = '|<image/\s+([^>\s]+)([^>]*)>((?:(?!<image).)*?)</image>|si';
		$pattern[] = '|<image\s+([^>\s]+)([^>]*)>((?:(?!<image).)*?)</image>|si';
		$pattern[] = '|<image\s+(\w{3,6}://[^>\s]+)([^>]*)>|i';
		$pattern[] = '|<image/\s+([^>\s]+)([^>]*)>|i';
		$pattern[] = '|<image\s+([^>\s]+)([^>]*)>|i';
		$replacement[] = '<img alt="\\3" title="\\3" border="0" src="\\1"\\2>';
		$replacement[] = '<img alt="\\3" title="\\3" border="0" src="'.$ezin_url.'\\1"\\2>';
		$replacement[] = '<img alt="\\3" title="\\3" border="0" src="'.$repository_url.'\\1"\\2>';
		$replacement[] = '<img alt="" title="" border="0" src="\\1"\\2>';
		$replacement[] = '<img alt="" title="" border="0" src="'.$ezin_url.'\\1"\\2>';
		$replacement[] = '<img alt="" title="" border="0" src="'.$repository_url.'\\1"\\2>';
	} /* }}} */
	if (strpos($mask, 'L') !== false) { /* {{{ */
		$pattern[] = '|<link\s+(\w{3,6}://[^>\s]+)([^>]*)>((?:(?!<link).)*?)</link>|si';
		$pattern[] = '|<link\s+/([^>\s]*)([^>]*)>((?:(?!<link).)*?)</link>|si';
		$pattern[] = '|<link\s+([^>\s]+)([^>]*)>((?:(?!<link).)*?)</link>|si';
		$pattern[] = '|<link\s+(\w{3,6}://[^>\s]+)([^>]*)>|i';
		$pattern[] = '|<link\s+/([^>\s]*)([^>]*)>|i';
		$pattern[] = '|<link\s+([^>\s]+)([^>]*)>|i';
		$replacement[] = '<a href="\\1"\\2>\\3</a>';
		$replacement[] = '<a href="'.$ezin_url.'\\1"\\2>\\3</a>';
		$replacement[] = '<a href="'.$repository_url.'\\1"\\2>\\3</a>';
		$replacement[] = '<a href="\\1"\\2>\\1</a>';
		$replacement[] = '<a href="'.$ezin_url.'\\1"\\2>\\1</a>';
		$replacement[] = '<a href="'.$repository_url.'\\1"\\2>\\1</a>';
	} /* }}} */
	if (strpos($mask, 'E') !== false) { /* {{{ */
		$pattern[] = '|<extern-link\s+(\w{3,6}://[^>\s]+)([^>]*)>((?:(?!<extern-link).)*?)</extern-link>|si';
		$pattern[] = '|<extern-link\s+/([^>\s]*)([^>]*)>((?:(?!<extern-link).)*?)</extern-link>|si';
		$pattern[] = '|<extern-link\s+([^>\s]+)([^>]*)>((?:(?!<extern-link).)*?)</extern-link>|si';
		$pattern[] = '|<extern-link\s+(\w{3,6}://[^>\s]+)([^>]*)>|i';
		$pattern[] = '|<extern-link\s+/([^>\s]*)([^>]*)>|i';
		$pattern[] = '|<extern-link\s+([^>\s]+)([^>]*)>|i';
		$replacement[] = '<a target="_blank" href="\\1"\\2>\\3</a>';
		$replacement[] = '<a target="_blank" href="'.$ezin_url.'\\1"\\2>\\3</a>';
		$replacement[] = '<a target="_blank" href="http://\\1"\\2>\\3</a>';
		$replacement[] = '<a target="_blank" href="\\1"\\2>\\1</a>';
		$replacement[] = '<a target="_blank" href="'.$ezin_url.'\\1"\\2>\\1</a>';
		$replacement[] = '<a target="_blank" href="http://\\1"\\2>\\1</a>';
	} /* }}} */
	if (strpos($mask, 'R') !== false) { /* {{{ */
		$pattern[] = '|<repository-link\s+([^>\s]+)([^>]*)>((?:(?!<repository-link).)*?)</repository-link>|si';
		$pattern[] = '|<repository-link\s+([^>\s]+)([^>]*)>|i';
		$replacement[] = '<a target="_blank" href="'.$repository_url.'\\1"\\2>\\3</a>';
		$replacement[] = '<a target="_blank" href="'.$repository_url.'\\1"\\2>\\1</a>';
	} /* }}} */
	if (strpos($mask, 'I') !== false) { /* {{{ */
		$pattern[] = '|<image-link\s+([^>\s]+)([^>]*)>((?:(?!<image-link).)*?)</image-link>|si';
		$pattern[] = '|<image-link\s+([^>\s]+)([^>]*)>|i';
		$replacement[] = '<a target="_blank" href="'.$repository_url.'\\1"\\2>\\3</a>';
		$replacement[] = '<a target="_blank" href="'.$repository_url.'\\1"\\2>\\1</a>';
	} /* }}} */
	if (strpos($mask, 'M') !== false) { /* {{{ */
		$pattern[] = '|<mail-link\s+([^>\s]+)([^>]*)>((?:(?!<mail-link).)*?)</mail-link>|sie';
		$pattern[] = '|<mail-link\s+([^>\s]+)([^>]*)>|ie';
		//$replacement[] = '<a href="mailto:\\1"\\2>\\3</a>';
		//$replacement[] = '<a href="mailto:\\1"\\2>\\1</a>';
		$replacement[] = '\'<a href="mailto:\'.Platon::escape_email(\'\\1\', \'url\').\'"\\2>\\3</a>\'';
		$replacement[] = 'Platon::escape_email(\'\\1\', \'hypertext_link\', \'\\2\')';
	} /* }}} */
	if (strpos($mask, 'S') !== false) { /* {{{ */
		$sub_url = $link_file_type ? 'section.php?' : '?ezin_section_id=';
		$pattern[] = '|<section-link\s+([^>\s]+)([^>]*)>((?:(?!<section-link).)*?)</section-link>|si';
		$pattern[] = '|<section-link\s+([^>\s]+)([^>]*)>|i';
		$replacement[] = '<a href="'.$ezin_url.$sub_url.'\\1"\\2>\\3</a>';
		$replacement[] = '<a href="'.$ezin_url.$sub_url.'\\1"\\2>\\1</a>';
	} /* }}} */
	if (strpos($mask, 'A') !== false) { /* {{{ */
		$sub_url = $link_file_type ? 'article.php?' : '?ezin_article_id=';
		$pattern[] = '|<article-link\s+([^>\s]+)([^>]*)>((?:(?!<article-link).)*?)</article-link>|si';
		$pattern[] = '|<article-link\s+([^>\s]+)([^>]*)>|i';
		$replacement[] = '<a href="'.$ezin_url.$sub_url.'\\1"\\2>\\3</a>';
		$replacement[] = '<a href="'.$ezin_url.$sub_url.'\\1"\\2>\\1</a>';
	} /* }}} */
	if (count($pattern) > 0) {
		$str = preg_replace($pattern, $replacement, $str);
	}
} /* }}} */

function ezin_message_html_process(&$text) /* {{{ */
{
	global $ezin_cfg;
	if (! is_array($ezin_cfg['message_allowed_tags'])) {
		$ezin_cfg['message_allowed_tags'] = ezin_parse_allowed_tags($ezin_cfg['message_allowed_tags']);
	}
	$max_word_size = intval($ezin_cfg['message_max_word_size']);
	$ar            = preg_split('|(<[^<>]+>)|', $text, -1, PREG_SPLIT_DELIM_CAPTURE);
	$text          = '';
	for ($k = 0; $k < count($ar); $k++) {
		if ($k & 1) {
			$ar2   = preg_split('|\s+|', trim(substr($ar[$k], 1, strlen($ar[$k]) - 2)));
			$tag   = strtolower(trim($ar2[0]));
			$opt   = array();
			$close = $tag[0] == '/';
			$close && $tag = substr($tag, 1);
			if (in_array($tag, array_keys($ezin_cfg['message_allowed_tags']))) {
				if ($close) {
					$text .= "</$tag>";
					continue;
				} 
				$text .= '<'.$tag;
				foreach ($ar2 as $attr) {
					@list($attr_name, $attr_value) = explode('=', $attr, 2);
					$attr_name  = strtolower($attr_name);
					$attr_value = str_replace('"', '', $attr_value);
					if (in_array($attr_name, $ezin_cfg['message_allowed_tags'][$tag]['allowed'])) {
						$text .= " $attr_name=\"$attr_value\"";
					}
				}
				count($ezin_cfg['message_allowed_tags'][$tag]['append'])
					&& $text .= ' '.join(' ', $ezin_cfg['message_allowed_tags'][$tag]['append']);
				$text .= '>';
				continue;
			}
		}
		if ($max_word_size > 0) {
			$last_space_pos = -1;
			$cur_space_pos  =  0;
			while ($cur_space_pos + 1 < strlen($ar[$k])) {
				$cur_space_pos = strpos($ar[$k], ' ', $last_space_pos + 1);
				$cur_space_pos === false && $cur_space_pos = strlen($ar[$k]);
				if ($cur_space_pos - $last_space_pos - 1 > $max_word_size) {
					$ar[$k]  = substr($ar[$k], 0, $last_space_pos + $max_word_size + 1)
					.' '.substr($ar[$k], $last_space_pos + $max_word_size + 1);
					$cur_space_pos = $last_space_pos = $last_space_pos + $max_word_size + 1;
				} else {
					$last_space_pos = $cur_space_pos;
				}
			}
		}

		$text .= htmlspecialchars($ar[$k]);
	}

	// TODO: optimalization using arrays
	$text = str_replace('&amp;#8220;', '&#8220;', $text); // left quotes
	$text = str_replace('&amp;#8221;', '&#8221;', $text); // right quotes
	$text = str_replace('&amp;#8211;', '&#8211;', $text); // nice long dash
	$text = str_replace('&amp;#8230;', '&#8230;', $text); // hellip (...)
	$text = str_replace('&amp;#8222;', '&#8222;', $text); // ?
} /* }}} */

function ezin_parse_allowed_tags($str) /* {{{ */
{
	$ret    = array();
	$tmp_ar = preg_split('|\s+|', $str);
	foreach ($tmp_ar as $val) {
		$tmp_ar2  = preg_split('|([:;])|', $val, -1, PREG_SPLIT_DELIM_CAPTURE);
		$tag_opts = array('allowed' => array(), 'append' => array());
		$tag_name = trim($tmp_ar2[0]);
		if ($tag_name == '')
			continue;
		for ($i = 1; $i < count($tmp_ar2); $i++) {
			$mode = $tmp_ar2[$i];
			$i++;
			$opt = trim($tmp_ar2[$i]);
			if ($opt == '')
				continue;
			$tag_opts[$mode == ';' ? 'append' : 'allowed'][] = $opt;
		}
		$ret[$tag_name] = $tag_opts;
	}
	return $ret;
} /* }}} */

/*
 * Register access function
 */

function ezin_register_access($destroy = 1, $print = 1) /* {{{ */
{
	global $ezin_cfg;
	global $ezin_cgi;
	global $ezin_weblog;

	$start_time = Platon::get_mtime();
	$ezin_weblog->enableRawLog();
	$ezin_weblog->registerAccess('ezin:'.$ezin_cfg['abbreviation'] .':'.$ezin_cgi['section_id']
			. (! empty($ezin_cgi['article_id']) ? ','.$ezin_cgi['article_id'] : ''));

	if ($destroy) {
		$ezin_weblog->destroy();
	}

	if ($print) {
		printf('<!-- weblog time: %2f -->'."\n", Platon::get_mtime() - $start_time);
	}
} /* }}} */

/*
 * Weblog counts getting
 */

function ezin_get_weblog_count($count_type, $id) /* {{{ */
{
	global $ezin_cfg;
	global $ezin_weblog;
	$weblog_count  = 0;

	if ($ezin_weblog != false) {

		$section_type  = ! strncmp($count_type, 'section', 7);
		$accesses_type = substr($count_type, -14) == 'accesses_count';
		$raw_type      = ! strncmp(substr($count_type, -18), 'raw', 3);
		$value         = 'ezin:'.$ezin_cfg['abbreviation'].':';
		$value        .= ! $section_type ? '%,'.$id : ($raw_type ? $id.',%' : $id);
		//echo 'value = '.$value;

		$ezin_weblog->enableRawLog();
		for ($j = 0; $j < 2; $j++) {
			$weblog_count += ($accesses_type
					? $ezin_weblog->getAccessesCount(array('value' => $value))
					: $ezin_weblog->getSessionsCount(array('value' => $value)));
			if (is_object($weblog_count)) { /* PEAR error object */
				$weblog_count = 0;
				break;
			}
			$ezin_weblog->disableRawLog();
		}
	}

	return $weblog_count;
} /* }}} */

/*
 * Link changing
 */

function ezin_get_link($type = 'S', $id = 0, $full_url = 0, $lang = '', $force_lang = 0) /* {{{ */
{
	global $ezin_cfg;

	$url_prefix = $full_url ? 'http://'.$_SERVER['HTTP_HOST'] : '';

	if (! empty($id)) {
		$append_params = '';
		switch ($type) {
			default:
			case 'A-show-all': $script = 'article'; $append_params = '&ezin_show_all=1'; break;
			case 'A-add-new':  $script = 'article'; $append_params = '&ezin_add_new=1';  break;
			case 'A': $script = 'article'; break;
			case 'S': $script = 'section'; break;
			case 'P': $script = 'print';   break;
			case 'M': $script = 'message'; break;
		}
		if ($ezin_cfg['link_file_type']) {
			return $url_prefix.$ezin_cfg['url']
				.$script.'.php'
				.'?'.htmlspecialchars($id)
				.htmlspecialchars($append_params)
				.($force_lang && $lang != '' ? htmlspecialchars('&ezin_lang='.$lang) : '');
		}

		return $url_prefix.$ezin_cfg['url'].'?ezin_'
			.($script == 'print' ? 'article' : $script)
			.(intval($id) > 0 ? '_id=' : '_key=').htmlspecialchars($id)
			.htmlspecialchars($append_params)
			.($force_lang && $lang != '' ? htmlspecialchars('&ezin_lang='.$lang) : '')
			.($script == 'print' ? htmlspecialchars('&ezin_print=1') : '');
	}

	return $url_prefix.$ezin_cfg['url']
	.($lang != '' ? htmlspecialchars('?ezin_lang='.$lang) : '');
} /* }}} */

function ezin_adjust_link($link, $full_url = 0, $lang_var = 0) /* {{{ */
{
	if (! (strncmp($link, 'http://', 7) && strncmp($link, 'https://', 8)))
		return $link;

	global $ezin_cgi;
	global $ezin_cfg;

	/* NOTE/TODO: There was some errors in the past:
	 * error_message: Undefined index:  HTTP_HOST */
	$url_prefix = $full_url ? 'http://'.$_SERVER['HTTP_HOST'] : '';
	if ($link[0] == '/') {
		$link = $ezin_cfg['url'] . substr($link, 1);
	} else if ($full_url) {
		if (($pos = strrpos($_SERVER['REQUEST_URI'], '/')) !== false) {
			$url_prefix .= substr($_SERVER['REQUEST_URI'], 0, $pos + 1);
		}
	}
	$link = $url_prefix . $link;
	if ($lang_var && strlen(@$ezin_cgi['lang']) > 0) {
		$link .= strpos($link, '?') === false ? '?' : '&';
		$link .= 'ezin_lang='.$ezin_cgi['lang'];
	}
	return $link;
} /* }}} */

/*
 * Elements link handling
 */

function ezin_handle_section_link($section_link) /* {{{ */
{
	global $HTTP_SERVER_VARS;
	$relative_link = ezin_adjust_link($section_link, 0);
	$request_uri   = $HTTP_SERVER_VARS['REQUEST_URI'];

	if (strncmp($request_uri, $relative_link, strlen($relative_link))) {
		// current URL doesn't compliant section link, make HTTP/1.1 redirect
		$absolute_link = ezin_adjust_link($section_link, 1, 1 /* not needed, just for sure */); 
		Platon::redirect($absolute_link);
	}
} /* }}} */

function ezin_section_link_process(&$section, $prefix = '') /* {{{ */
{
	$section[$prefix.'section_id2'] = empty($section[$prefix.'section_access_key'])
		? $section[$prefix.'section_id']
		: $section[$prefix.'section_access_key'];
	$section[$prefix.'section_link'] = empty($section[$prefix.'section_link'])
		? ezin_get_link('S', $section[$prefix.'section_type'] != 'global'
				? $section[$prefix.'section_id2'] : 0)
		: ezin_adjust_link($section[$prefix.'section_link']);
} /* }}} */

function ezin_section_counts_process(&$section) /* {{{ */
{
	$weblog_ar = array();
	foreach (array('section_sessions_count', 'section_accesses_count',
				'section_raw_sessions_count', 'section_raw_accesses_count')
			as $weblog_val) { /* raw counts must go after normal counts */
		if (intval(@$section[$weblog_val]) < 0) {
			$weblog_ar[$weblog_val] = ezin_get_weblog_count($weblog_val, $section['section_id']);
			if (preg_match('/^section_raw_(.*)$/', $weblog_val, $matches)) {
				$weblog_ar[$weblog_val] += $section['section_'.$matches[1]];
			}
			$section[$weblog_val] = $weblog_ar[$weblog_val];
		}
	}
	if (count($weblog_ar) > 0) {
		ezin_set_section_counts($section['section_id'], $weblog_ar);
	}
} /* }}} */

function ezin_article_link_process(&$article) /* {{{ */
{
	$article['article_id2'] = empty($article['article_access_key'])
		? $article['article_id']
		: $article['article_access_key'];
	$article['article_add_new_link']  = ezin_get_link('A-add-new', $article['article_id2']);
	$article['article_show_all_link'] = ezin_get_link('A-show-all', $article['article_id2']);
	$article['article_link']          = ezin_get_link('A', $article['article_id2']);
	$article['article_print_link']    = ezin_get_link('P', $article['article_id2']);
} /* }}} */

function ezin_article_counts_process(&$article) /* {{{ */
{
	$weblog_ar = null;
	foreach (array('article_sessions_count', 'article_accesses_count') as $weblog_val) {
		if (intval(@$article[$weblog_val]) < 0) {
			$weblog_ar[$weblog_val]    = ezin_get_weblog_count($weblog_val, $article['article_id']);
			$article[$weblog_val] = $weblog_ar[$weblog_val];
		}
	}
	if (isset($weblog_ar)) {
		ezin_set_article_counts($article['article_id'], $weblog_ar);
	}
	if (intval(@$article['article_message_count']) < 0) {
		$message_count = ezin_get_message_count($article['article_id']);
		ezin_update_message_count($article['article_id'], 0, $message_count);
		$article['article_message_count'] = $message_count;
	}
} /* }}} */

/*
 * Message process function
 */

function ezin_message_process($full, $message_id, $level = 0) /* {{{ */
{
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_cgi;
	global $ezin_tmpl_key;
	$filename  = $ezin_sys['tmpl']['message_'.($full?'full_':'').'item'];
	$filename .= $ezin_tmpl_key;
	$items     = '';
	$item      = new HTML_Template_Platon($filename);
	$messages  = ezin_get_message_list($ezin_cgi['article_id'], $message_id);

	for ($i = 0; $i < count($messages); $i++) {
		if ($ezin_cgi['output'] == 'message_selected_list'
				&& is_array($ezin_cgi['message_id'])
				&& ! in_array($messages[$i]['message_id'], $ezin_cgi['message_id'])) {
			// skip this message
			// note that you cannot use `continue' here
		} else {
			$item->reload();
			$messages[$i]['message_date']     = date($ezin_cfg['format_date'], $messages[$i]['message_timestamp']);
			$messages[$i]['message_datetime'] = date($ezin_cfg['format_datetime'], $messages[$i]['message_timestamp']);
			$messages[$i]['message_level']    = $level;
			$messages[$i]['message_link']     = ezin_get_link('M', $messages[$i]['message_id']);
			$messages[$i]['message_author']   = htmlspecialchars($messages[$i]['message_author']);
			$messages[$i]['message_homepage'] = htmlspecialchars($messages[$i]['message_homepage']);
			$messages[$i]['message_subject']  = htmlspecialchars($messages[$i]['message_subject']);
			ezin_message_html_process($messages[$i]['message_message']);
			ezin_process_text_flags($messages[$i]['message_message'], 'message', 1);
			/* if atitle is empty, set it to title,
			   but if title is empty, remains it unchanged */
			if (empty($messages[$i]['message_article_atitle']) && ! empty($messages[$i]['message_article_title'])) {
				$messages[$i]['message_article_atitle'] = $messages[$i]['message_article_title'];
			}
			$item->set($messages[$i]);
			$items .= $item->getCode();
		}
		$items .= ezin_message_process($full, $messages[$i]['message_id'], $level + 1);
	}
	return $items;
} /* }}} */

/*
 * Feedback e-mail sending
 */
function ezin_feedback_send_notify_email($data) /* {{{ */
{
	global $ezin_sys, $ezin_cfg;
	if (! isset($ezin_cfg['feedback_email']) || strlen($ezin_cfg['feedback_email']) <= 0) {
		return true;
	}
	$from     = '';
	$subject  = '';
	$encoding = '';
	$message  = '';
	$tmpl = ezin_load_template($ezin_sys['tmpl']['e-mail']['feedback'], 1);	
	if ($tmpl != false) {
		$tmpl = new HTML_Template_Platon($ezin_sys['tmpl']['e-mail']['feedback']);
	}
	if ($tmpl == false || $tmpl->isVar('content')) {
		$message .= 'Dear administrator of ';
		$message .= $ezin_sys['software']['name']." on $ezin_cfg[name]\n\n";
		$message .= 'Feedback form was submited on your portal at server time ';
		$message .= date($ezin_cfg['format_datetime']) . '.';
		$message .= "\nFull information follows on the next lines.\n\n";
		foreach ($data as $key => $val) {
			$val = trim($val);
			if (strpos($val, "\n") === false) {
				$message .= sprintf("* %s: %s\n", $key, $val);
			} else {
				$message .= sprintf("* %s:\n\n%s\n\n", $key, $val);
			}
		}
		$message .= "\nHoping the message above is a good one.";
		$message .= "\n\nYours,\n\n";
		$message .= $ezin_sys['software']['name']." on $ezin_cfg[name]\n\n";
		if ($tmpl != false) {
			$tmpl->set('content', $message);
		}
	}
	if ($tmpl != false) {
		foreach ($data as $key => $val) {
			$tmpl->set("feedback_$key", $val);
		}
		ezin_substitute_basic_properties($tmpl);
		ob_start();
		@$tmpl->Output();
		$message = ob_get_contents();
		ob_end_clean();
		$message = ezin_email_template_parse($message, $from, $subject, $encoding);
	}
	empty($subject) && $subject = sprintf('%s: feedback', $ezin_cfg['name']);
	return ezin_email_send($ezin_cfg['feedback_email'], $from, $subject, $encoding, $message);
} /* }}} */

/*
 * Template load function
 */

function ezin_load_template($template_key, $test_only = 0) /* {{{ */
{
	global $ezin_sys;
	global $ezin_cfg;
	global $ezin_cgi;

	$theme_dir = $ezin_sys['dir']['root'].'/'.$ezin_sys['dir']['themes'].'/'.$ezin_cfg['theme']['name'].'/';
	if (isset($ezin_cgi['lang']) && $ezin_cgi['lang'] != '') {
		$template_key .= $ezin_sys['tmpl']['delim'].$ezin_cgi['lang'];
	}
	while (1) {
		$filename = $theme_dir . $template_key . $ezin_sys['tmpl']['delimext'];
		if (@file_exists($filename) && $f = @fopen($filename, 'r')) {
			if ($test_only) {
				fclose($f);
				return true;
			}
			$data = '';
			while ($tmp_data = fread($f, 1024)) {
				$data .= $tmp_data;
			}
			fclose($f);
			return $data;
		}
		if (($pos = strrpos($template_key, '.')) === false) {
			break;
		}
		$template_key = substr($template_key, 0, $pos);
	}
	return false;
} /* }}} */

function ezin_email_template_parse($message, &$from, &$subject, &$encoding) /* {{{ */
{
	$match = array();
	preg_match("/^(From:(.*?)[\r\n]+?)?(Subject:(.*?)[\r\n]+?)?(Charset:(.*?)[\r\n]+?)?(.*?)$/is", $message, $match); 
	$message  = ( isset($match[7]) ) ? trim($match[7]) : '';
	$from     = ( trim($match[2]) != '' ) ? trim($match[2]) : '';
	$subject  = ( trim($match[4]) != '' ) ? trim($match[4]) : '';
	$encoding = ( trim($match[6]) != '' ) ? trim($match[6]) : '';
	return $message;
} /* }}} */

// DELETE ME IN SPRING 2005
function ezin_email_send_old($emails, $from, $subject, $encoding, $message) /* {{{ */
{
	global $ezin_cfg;
	$message .= "\n\n--\n";
	$message .= "phpMyEzin - e-zin content management system\n";
	$message .= 'http://platon.sk/projects/phpMyEzin/';
	empty($from)     && $from = sprintf('"phpMyEzin on %s" <webmaster@%s>', $ezin_cfg['name'], $_SERVER['SERVER_NAME']);
	empty($subject)  && $subject = sprintf('%s issues', $ezin_cfg['name']);
	empty($encoding) && $encoding = $ezin_cfg['encoding'];
	empty($encoding) && $encoding = 'iso-8859-1';
	$headers = array(
			'MIME-Version: 1.0',
			'Content-type: text/plain; charset='.$encoding,
			'Content-transfer-encoding: 8bit',
			'Date: '.gmdate('D, d M Y H:i:s', time()).' UT',
			'From: '.$from);
	$headers = join("\n", $headers);
	if (! is_array($emails)) {
		$emails  = preg_split('|\s+|', $emails);
	}
	$ret = true;
	foreach ((array) $emails as $email) {
		$email = trim($email);
		if (empty($email) || strpos($email, '@') === false) {
			continue;
		}
		$ret &= @mail($email, $subject, $message, $headers);
	}
	return $ret;
} /* }}} */

function ezin_email_send($emails, $from, $subject, $encoding, $message) /* {{{ */
{
	global $ezin_cfg, $ezin_sys;

	include_once 'Mail.php';
	include_once 'Mail/mime.php';

	$message .= "\n\n--\n";
	$message .= $ezin_sys['software']['name'].' - '.$ezin_sys['software']['description'];
	$message .= "\nversion ".$ezin_sys['software']['version'];
	$message .= "\n\n\"".$ezin_sys['software']['motto'].'" (tm)';
	$message .= "\n\n".$ezin_sys['software']['url'];
	empty($subject)  && $subject  = sprintf('%s issues', $ezin_cfg['name']);
	empty($encoding) && $encoding = $ezin_cfg['encoding'];
	empty($encoding) && $encoding = 'iso-8859-1';
	empty($from)     && $from = sprintf('"%s on %s" <webmaster@%s>',
			$ezin_sys['software']['name'], $ezin_cfg['name'], $_SERVER['SERVER_NAME']);
	if (! is_array($emails)) {
		$emails = preg_split('|\s+|', $emails);
	}
	foreach (array_keys($emails) as $key) {
		$emails[$key] = trim($emails[$key]);
		if (empty($emails[$key]) || strpos($emails[$key], '@') === false) {
			unset($emails[$key]);
		}
	}
	
	$mime = new Mail_mime();
	$mime->setTXTBody($message);

	$atts = array(); // this is prepared for atts sending
	if (count($atts) > 0) {
		foreach ($atts as $file => $mime_type) {
			if (strlen($mime_type) <= 0) {
				$mime_type = 'application/octet-stream';
			}
			$mime->addAttachment($file, $mime_type, basename($file));
		}
	}
	$message = $mime->get(array(
				'text_charset' => $encoding,
				'html_charset' => $encoding,
				'head_charset' => $encoding
				));
	$headers = array(
			//'MIME-Version: 1.0',
			//'Content-type: text/plain; charset='.$encoding,
			//'Content-transfer-encoding: 8bit',
			//'Date' => gmdate('D, d M Y H:i:s', time()).' UT',
			'From'     => $from,
			'To'       => join($emails, ', '),
			'Subject'  => $subject,
			'X-Mailer' => $ezin_sys['software']['name'].'/'.$ezin_sys['software']['version'],
			'X-Mail-Prefered-Method' => $ezin_cfg['mail_method'],
			'X-Mail-Used-Method'     => 'undefined',
			'X-Mail-Failed-Methods'  => 'none',
			);
	$headers = $mime->headers($headers);
	$methods = array($ezin_cfg['mail_method'], 'mail', 'sendmail', 'smtp');
	$methods = array_unique($methods);
	$params  = array(
			'sendmail_path' => '/usr/sbin/sendmail',
			'host'          => '127.0.0.1',
			'port'          => 25
			);
	if (strlen(trim($ezin_cfg['mail_sendmail_path'])) > 0) {
		$params['sendmail_path'] = trim($ezin_cfg['mail_sendmail_path']);
	}
	if (strlen(trim($ezin_cfg['mail_smtp_host_port'])) > 0) {
		@list($host, $port) = explode(':', trim($ezin_cfg['mail_smtp_host_port']));
		strlen($host) > 0 && $params['host'] = $host;
		strlen($port) > 0 && $params['port'] = $port;
	}
	PEAR::pushErrorHandling(PEAR_ERROR_RETURN);
	foreach ($methods as $method) {
		$headers['X-Mail-Used-Method'] = $method;
		$mail = &Mail::factory($method, $params);
		$ret  = @$mail->send($emails, $headers, $message);
		if ($ret === true) {
			//echo "method <b>$method</b> returned <b>true</b><br>";
			return true;
		}
		//echo "method <b>$method</b> returned <b>false</b><br>";
		unset($mail);
		@$headers['X-Mail-Failed-Methods'] .= ", $method";
	}
	PEAR::popErrorHandling();
	return false;
} /* }}} */

/*
 * Basic template substitutions
 */

function ezin_substitute_basic_properties(&$tmpl_obj) /* {{{ */
{
	global $ezin_sys, $ezin_cfg, $ezin_cgi;

	$tmpl_obj->set(array(
				// Configuration
				'ezin_name'      => @$ezin_cfg['name'],
				'ezin_url'       => @$ezin_cfg['url'],
				'ezin_encoding'  => @$ezin_cfg['encoding'],
				'ezin_charset'   => @$ezin_cfg['encoding'],
				'ezin_count'     => 'N/A',
				'ezin_theme_dir' => @$ezin_cfg['theme']['dir'],
				'ezin_theme_url' => @$ezin_cfg['theme']['url'],
				'ezin_upload_dir' => @$ezin_cfg['upload']['dir'],
				'ezin_upload_url' => @$ezin_cfg['upload']['url'],
				'ezin_debug'	 => intval(@$ezin_cfg['debug_level']),
				'ezin_datetime'  => date($ezin_cfg['format_datetime']),
				'ezin_date'      => date($ezin_cfg['format_date']),
				'ezin_time'      => sprintf('%.2f', Platon::get_mtime() - @$ezin_sys['time']['start']),

				// Ezin software
				'ezin_software_name'        => @$ezin_sys['software']['name'],
				'ezin_software_version'     => @$ezin_sys['software']['version'],
				'ezin_software_description' => @$ezin_sys['software']['description'],
				'ezin_software_motto'       => @$ezin_sys['software']['motto'],
				'ezin_software_url'         => @$ezin_sys['software']['url'],
				'ezin_software_signature'   => @$ezin_sys['software']['name']
										.'/'.@$ezin_sys['software']['version'],

				// CGI parameters
				'ezin_cgi_input'  => @$ezin_cgi['input'],
				'ezin_cgi_output' => @$ezin_cgi['output'],
				'ezin_message_id' => @$ezin_cgi['message_id'],
				'ezin_article_id' => @$ezin_cgi['article_id'],
				'ezin_section_id' => @$ezin_cgi['section_id'],
				'ezin_lang'       => @$ezin_cgi['lang']
					));

				// Language variables
				if (isset($ezin_cgi['lang_bind']) && is_array($ezin_cgi['lang_bind'])) {
					foreach ($ezin_cgi['lang_bind'] as $ezin_lang_key => $ezin_lang_val) {
						$tmpl_obj->set('ezin_lang_'.$ezin_lang_key,
								intval($ezin_lang_key == $ezin_cgi['lang']));
						$tmpl_obj->set('ezin_lang_'.$ezin_lang_key.'_link',
								$ezin_lang_key == $ezin_cgi['lang'] || strlen($ezin_lang_val) <= 0
								? '#' : $ezin_lang_val);
					}
				}
} /* }}} */

/*
 * Input parameters parsing
 * TODO: this should be probably completely rewritten
 */

function ezin_cgi_parse() /* {{{ */
{
	global $ezin_cgi;
	global $ezin_cfg;
	global $HTTP_COOKIE_VARS;

	/* $lang_init schema

	   0 - uninitialized
	   1 - initialized by CGI
	   2 - initialized by cookie
	   3 - initialized by negotiation
	 */

	$expire_time = time() + (3600 * 24 * 30 * 12 * 2); // two years
	$lang_init   = 0;
	$lang_cookie = strtolower(@$HTTP_COOKIE_VARS['ezin_lang']);
	if (is_array($ezin_cfg['languages'])) {
		$lang_init = 1;
		$lang = strtolower(Platon::parse_id_param('ezin_lang', false));
		if (! in_array($lang, $ezin_cfg['languages'])) {
			$lang_init = 2;
			$lang = $lang_cookie;
			if (! in_array($lang, $ezin_cfg['languages'])) {
				$lang_init = 3;
				$lang = Platon::negotiate_language($ezin_cfg['languages']);
			}
		}
	} else {
		//$lang_get_post = $lang_cookie = '';
		$lang = '';
	}

	switch ($ezin_cgi['input']) {
		case 'admin':
			$ezin_cgi['section_id'] = 0;
			$ezin_cgi['article_id'] = 0;
			$ezin_cgi['message_id'] = 0;
			return;
		case 'section':
			if (! isset($ezin_cgi['section_id']) || empty($ezin_cgi['section_id'])) {
				$ezin_cgi['section_id'] = Platon::parse_id_param('id', $ezin_cgi['query_string']);
				if (empty($ezin_cgi['section_id'])) {
					$ezin_cgi['section_id'] = Platon::parse_id_param('key', $ezin_cgi['query_string']);
				}
			}
			break;
		case 'article':
		case 'print':
			if (! isset($ezin_cgi['article_id']) || empty($ezin_cgi['article_id'])) {
				$ezin_cgi['article_id'] = Platon::parse_id_param('id', $ezin_cgi['query_string']);
				if (empty($ezin_cgi['article_id'])) {
					$ezin_cgi['article_id'] = Platon::parse_id_param('key', $ezin_cgi['query_string']);
				}
			}
			break;
		case 'message':
			if (! isset($ezin_cgi['message_id']) || empty($ezin_cgi['message_id'])) {
				$ezin_cgi['message_id'] = Platon::parse_id_param('id', $ezin_cgi['query_string']);
			}
			break;
	}

	if (! isset($ezin_cgi['message_id']) || empty($ezin_cgi['message_id'])) {
		$ezin_cgi['message_id'] = Platon::parse_id_param('ezin_message_id', false);
	}
	if (! isset($ezin_cgi['article_id']) || empty($ezin_cgi['article_id'])) {
		$ezin_cgi['article_id'] = Platon::parse_id_param('ezin_article_id', false);
		if (empty($ezin_cgi['article_id'])) {
			$ezin_cgi['article_id'] = Platon::parse_id_param('ezin_article_key', false);
		}
	}
	if (! isset($ezin_cgi['section_id']) || empty($ezin_cgi['section_id'])) {
		$ezin_cgi['section_id'] = Platon::parse_id_param('ezin_section_id', false);
		if (empty($ezin_cgi['section_id'])) {
			$ezin_cgi['section_id'] = Platon::parse_id_param('ezin_section_key', false);
		}
	}

	$ar = ezin_validate_IDs(array(
				'section_id' =>	$ezin_cgi['section_id'],
				'article_id' => $ezin_cgi['article_id'],
				'message_id' => $ezin_cgi['message_id'],
				'lang'       => $lang));
	$lang_new = $ar['lang'];
	$lang_tbl_type = 'S';
	$lang_tbl = ezin_get_bind_lang_table_wrapper($lang_tbl_type,
			$ar['section_bind_lang'], $ar['article_bind_lang']);

	while ($ar['section_id'] + $ar['article_id'] + (int)$ar['message_id'] <= 0) {
		global $ezin_cfg;
		$ezin_cgi['output'] = 'notfound';
		$redirect_data = $ezin_cfg[
			$ezin_cgi['input'] == 'index' ? 'redirect_index' : 'redirect_others'];
		$redirect_http = $redirect_data[0] == '+';
		$redirect_data = $redirect_http ? substr($redirect_data, 1) : $redirect_data;
		$redirect_type = $redirect_data[0];
		$redirect_id   = substr($redirect_data, 1);
		$redirect_section_id = $redirect_type == 'S' ? $redirect_id : 0;
		$redirect_article_id = $redirect_type == 'A' ? $redirect_id : 0;

		$ar = ezin_validate_IDs(array(
				'section_id' => $redirect_section_id,
				'article_id' => $redirect_article_id,
				'message_id' => 0,
				'lang'       => $lang));
		$lang_new = $ar['lang'];
		$lang_tbl = ezin_get_bind_lang_table_wrapper($lang_tbl_type,
				$ar['section_bind_lang'], $ar['article_bind_lang']);

		/* === FUNNY CODE (c) Nepto === */
		if ($lang != '' && $lang_new != '' && $lang_init < 3 /* !!! */ && $lang != $lang_new) {
			if (! empty($lang_tbl[$lang])) {
				if (intval($ar['article_bind_lang']) > 0) {
					$ar = ezin_validate_IDs(array(
								'section_id' => $redirect_section_id,
								'article_id' => $lang_tbl[$lang],
								'message_id' => 0,
								'lang'       => $lang));
					$redirect_id = $ar['article_id'];
				} else {
					$redirect_id = $ar['section_id'] = $lang_tbl[$lang];
				}
				$lang_new = $lang;
			}
		}
		/* ============================ */
		if ($ar['section_id'] + $ar['article_id'] + (int)$ar['message_id'] > 0) {
			if ($redirect_http) {
				Platon::redirect(ezin_get_link($redirect_type, $redirect_id, 1),
						true /* temporary redirect */);
			}
			$ezin_cgi['output'] = $redirect_type == 'S' ? 'section' : 'article';
		}
		break;
	}

	/* === FUNNY CODE (c) Nepto === */
	if ($lang != '' && $lang_new != '' && $lang_init < 2 && $lang != $lang_new) {
		if (! empty($lang_tbl[$lang])) {
			if (intval($ar['article_bind_lang']) > 0) {
				$ar = ezin_validate_IDs(array(
							'section_id' => $ezin_cgi['section_id'],
							'article_id' => $lang_tbl[$lang],
							'message_id' => 0,
							'lang'       => $lang));
			} else {
				$ar['section_id'] = $lang_tbl[$lang];
			}
			$lang_new = $lang;
		}
	}
	/* ============================ */

	if (is_array($ezin_cfg['languages'])) {
		$ezin_cgi['lang'] = $lang_new != '' ? $lang_new : $lang;
		if ($ezin_cgi['lang'] != $lang_cookie) {
			setcookie('ezin_lang', $ezin_cgi['lang'], $expire_time, $ezin_cfg['url']);
		}
	} else {
		$ezin_cgi['lang'] = '';
	}

	$ezin_cgi['message_id'] = $ar['message_id'];
	$ezin_cgi['article_id'] = $ar['article_id'];
	$ezin_cgi['section_id'] = $ar['section_id'];
	$ezin_cgi['article_id2'] = empty($ar['article_access_key'])
		? $ar['article_id'] : $ar['article_access_key'];
	$ezin_cgi['section_id2'] = empty($ar['section_access_key'])
		? $ar['section_id'] : $ar['section_access_key'];

	if (! empty($ezin_cgi['article_id'])) {
		$ezin_cgi['output'] = 'article';

		if (! empty($ezin_cgi['message_id'])) {
			$ezin_cgi['output'] = 'message';
		}

		$ezin_save_message = Platon::parse_id_param('ezin_save_message', false);
		if (! empty($ezin_save_message)) {
			foreach (array('author', 'email', 'homepage', 'subject', 'message') as $var) {
				$ezin_cgi["message_reply_$var"]
					= trim(stripslashes(Platon::parse_id_param("ezin_$var", false)));
			}

			if (! preg_match('/^.+(:\/\/|:).+\..+$/', $ezin_cgi['message_reply_homepage'])) {
				if (preg_match("/^.+\..+$/", $ezin_cgi['message_reply_homepage'])) {
					$ezin_cgi['message_reply_homepage'] = 'http://'
						. $ezin_cgi['message_reply_homepage'];
				} else {
					$ezin_cgi['message_reply_homepage'] = '';
				}
			}

			$result = true;
			if (ezin_check_message_flood()) {
				$ezin_cgi['message_save_flood_protection'] = 1;
				$result = false;
			}
			if (strlen($ezin_cgi['message_reply_subject']) <= 0) {
				$ezin_cgi['message_save_subject_missing'] = 1;
				$result = false;
			}
			if (strlen($ezin_cgi['message_reply_author']) <= 0) {
				$ezin_cgi['message_save_author_missing'] = 1;
				$result = false;
			}
			if (strlen($ezin_cgi['message_reply_message']) <= 0) {
				$ezin_cgi['message_save_message_missing'] = 1;
				$result = false;
			}
			if ($result) {
				$result = ezin_save_message(array(
							'parent'     => $ezin_cgi['message_id'],
							'article_id' => $ezin_cgi['article_id'], 
							'author'     => $ezin_cgi['message_reply_author'],
							'email'      => $ezin_cgi['message_reply_email'],
							'homepage'   => $ezin_cgi['message_reply_homepage'],
							'subject'    => $ezin_cgi['message_reply_subject'],
							'message'    => $ezin_cgi['message_reply_message']
							));
				if ($result == false) {
					$ezin_cgi['message_save_unknown_error'] = 1;
				} else {
					ezin_cache_clear('a'.@$ezin_cgi['article_id']);
					ezin_cache_clear('A'.@$ezin_cgi['article_id']);
				}
			}

			if ($result == false) {
				$ezin_cgi['output']             = 'message';
				$ezin_cgi['message_save_error'] = 1;
			} else {
				ezin_update_message_count($ezin_cgi['article_id']);
				$ezin_cgi['output']          = 'article';
				$ezin_cgi['message_id']      = 0;
				$ezin_cgi['message_save_ok'] = 1;

				setcookie('ezin_author_cookie',   $ezin_cgi['message_reply_author'],   $expire_time);
				setcookie('ezin_email_cookie',    $ezin_cgi['message_reply_email'],    $expire_time);
				setcookie('ezin_homepage_cookie', $ezin_cgi['message_reply_homepage'], $expire_time);

				if ($ezin_cfg['message_save_redirect']) {
					$request_uri = $_SERVER['REQUEST_URI'];
					$location = strpos($request_uri, '?') === false
						? ezin_get_link('A', $ezin_cgi['article_id'], 1, @$ezin_cgi['lang'])
						: 'http://'.$_SERVER['HTTP_HOST'].$request_uri;
					if (strlen(@$_POST['ezin_fragment']) > 0) {
						$location .= '#'.$_POST['ezin_fragment'];
					}
					Platon::redirect($location, true /* temporary */);
				}
			}
		}

		$ezin_show_selected = Platon::parse_id_param('ezin_show_selected', false);
		$ezin_show_all      = Platon::parse_id_param('ezin_show_all',      false);
		if (! empty($ezin_show_selected)) {
			$ezin_cgi['output'] = 'message_selected_list';
		}
		if (! empty($ezin_show_all)) {
			$ezin_cgi['output'] = 'message_full_list';
		}

		$ezin_add_new = Platon::parse_id_param('ezin_add_new', false);
		if (! empty($ezin_add_new)) {
			$ezin_cgi['output']     = 'message';
			$ezin_cgi['message_id'] = 0;
		}

		$ezin_print = Platon::parse_id_param('ezin_print', false);
		if (($ezin_cgi['output'] == 'article' && ! empty($ezin_print))
				|| $ezin_cgi['input'] == 'print') {
			$ezin_cgi['output'] = 'article_print';
		}
	}

	if (! empty($ezin_cgi['section_id'])) {
		$ezin_save_feedback = Platon::parse_id_param('ezin_save_feedback', false);
		if (! empty($ezin_save_feedback)) {
			$data = array();
			foreach ($_POST + $_GET as $key => $val) {
				if (in_array($key, array('ezin_save_feedback', 'ezin_fragment',
								'ezin_section_id', 'ezin_section_key'))) {
					continue;
				}
				if (preg_match('|^ezin_(.+)$|', $key, $matches)) {
					$data[$matches[1]] = stripslashes($val);
				}
			}
			ezin_feedback_send_notify_email($data);
			if (ezin_save_feedback($ezin_cgi['section_id'], $data)) {
				$ezin_cgi['feedback_save_ok'] = 1;
				if ($ezin_cfg['feedback_save_redirect']) {
					$request_uri = $_SERVER['REQUEST_URI'];
					$location = strpos($request_uri, '?') === false
						? ezin_get_link('S', $ezin_cgi['section_id'], 1, @$ezin_cgi['lang'])
						: 'http://'.$_SERVER['HTTP_HOST'].$request_uri;
					if (strlen(@$_POST['ezin_fragment']) > 0) {
						$location .= '#'.$_POST['ezin_fragment'];
					}
					Platon::redirect($location);
				}
			} else {
				$ezin_cgi['feedback_save_error'] = 1;
			}
		}
	}

	/* This part creates $ezin_cgi['lang_bind'] array, which will be used
	   for template language variables substitution. */
	if (is_array($ezin_cfg['languages'])) {
		$type = $ezin_cgi['output'] == 'section' ? 'S' : 'A';
		foreach ($ezin_cfg['languages'] as $val) {
			if (empty($lang_tbl[$val])) {
				$id = $type == 'S' ? $ezin_cgi['section_id2'] : $ezin_cgi['article_id2'];
				$ezin_cgi['lang_bind'][$val] = ezin_get_link($type, $id, 0, $val, 1);
			} else {
				$ezin_cgi['lang_bind'][$val] = ezin_get_link($lang_tbl_type, $lang_tbl[$val.'2'], 0, $val);
			}
		}
	} else {
		$ezin_cgi['lang_bind'] = '';
	}
} /* }}} */

/* Modeline for ViM {{{
 * vim: set ts=4:
 * vim600: fdm=marker fdl=0 fdc=0:
 * }}} */

?>
